/*
 * @(#)HTTPServer.cpp
 *
 * This file is part of webCDwriter - Network CD Writing.
 *
 * Copyright (C) 2002 Jrg P. M. Haeger
 *
 * webCDwriter is free software. See CDWserver.cpp for details.
 *
 * Jrg Haeger, 18.04.2002
 */

#include "Config.h"
#include "File.h"
#include "FileInputStream.h"
#include "HTTPRequest.h"
#include "HTTPResponse.h"
#include "HTTPServer.h"
#include "Log.h"
#include "PrintWriter.h"
#include "Server.h"
#include "Socket.h"
#include "Status.h"
#include "String.h"
#include "StringBuffer.h"
#include "Version.h"

HTTPServer::HTTPServer(Socket &socket): socket(socket) {
}

HTTPServer::~HTTPServer() {
}

HTTPRequest *HTTPServer::nextRequest() {
	return new HTTPRequest(socket.getInputStream());
}

StringBuffer &HTTPServer::replace(StringBuffer &buf,
		const char *key, String &by) {
	log.debug("HTTPServer", "replace %d", buf.length());
	while (1) {
		int pos = buf.indexOf(key);
		if (pos < 0)
			break;
		buf.delete_(pos, pos + strlen(key));
		buf.insert(pos, by.getBytes());
		log.debug("HTTPServer", "replace2 %d", buf.length());
	}
	return buf;
}

void HTTPServer::run() {
	log.putv("HTTPServer::run() start %d", socket.getOutputStream());
	PrintWriter writer(socket.getOutputStream());

	while (1) {
		HTTPRequest request = nextRequest();

		if (request.method.equalsIgnoreCase("GET")
			|| request.method.equalsIgnoreCase("HEAD")) {
			time_t t;
			time(&t);
			struct tm *tm = localtime(&t);
			log.putv("%s - "
				"%4d%02d%02d %02d:%02d:%02d"
				" - %s <%s> <%s>",
				socket.getInetAddress(),
				1900 + tm->tm_year, tm->tm_mon + 1, tm->tm_mday,
				tm->tm_hour, tm->tm_min, tm->tm_sec,
				request.method.getBytes(),
				request.uri.getBytes(),
				request.protocol.getBytes());

			if (request.uri.equals("/webCDcreator/4netscape.html")) {
				File file = new File(config.getConfigDir(),
					"/webCDcreator/4netscape/swing.jar");
				if (!file.exists())
					request.uri = "/webCDcreator/4netscape/swing.html";
			}

			if (request.uri.equals("/")
				|| request.uri.equals("/index.html"))
				request.uri = "/status.html";

			if (request.uri.indexOf("..") == -1)
			if (request.uri.startsWith("/webCDcreator/")
				|| request.uri.equals("/status.html")
				|| request.uri.equals("/logo.png")
				|| request.uri.equals("/favicon.ico")
				|| request.uri.startsWith("/rcdrecord/")
				|| request.uri.equals("/config.html"))
			try {
				serve(request, writer);
				log.putv("HTTPServer::run() OK");
				break;
			} catch (Exception *e) {
				HTTPResponse response = HTTPResponse::notFound();
				response.write(writer);
				delete e;
				break;
			}

			HTTPResponse response = HTTPResponse::notFound();
			response.write(writer);
			break;
		}
		else if (request.method.equalsIgnoreCase("CDWP")) {
			Server server(socket.getOutputStream());
			server.run();
			break;
		}
		else {
			HTTPResponse response = HTTPResponse::notImplemented();
			response.write(writer);
			break;
		}
	}

	log.putv("HTTPServer::run() end");
}

void HTTPServer::serve(HTTPRequest &request, PrintWriter &writer) {
	String &method = request.method;
	String &uri = request.uri;
	String &host = request.host;

	File file = new File(config.getConfigDir(), uri.getBytes());
	if (file.isDirectory())
		file = new File(file, "index.html");

	HTTPResponse response = new HTTPResponse();
	if (file.getPath().endsWith(".html")
			|| file.getPath().endsWith(".jnlp")) {
		FileInputStream in = new FileInputStream(file);
		StringBuffer workBuf;
		while (1) {
			char buf[4094];
			int n = in.read(buf, 0, sizeof buf);
			if (n <= 0)
				break;
			workBuf.append(buf, 0, n);
		}

		if (uri.equals("/status.html")) {
			String str = status.getStatus();
			replace(workBuf, "<!-- status -->", str);
		}

		if (uri.equals("/config.html")) {
			String str = config.toHTML();
			replace(workBuf, "<!-- config -->", str);
		}

		String configDir = config.getConfigDir();
		replace(workBuf, "@CONFIGDIR@", configDir);

		if (workBuf.indexOf("<body>") >= 0) {
			File f = new File(config.getConfigDir(), "header");
			FileInputStream in = new FileInputStream(f);
			char buf[16 * 1024];
			int n = in.read(buf, 0, sizeof buf);
			if (n > 0) {
				StringBuffer strBuf;
				strBuf.append(buf, 0, n);
				String str = new String(strBuf);
				replace(workBuf, "<body>", str);
			}

			f = new File(config.getConfigDir(), "footer");
			in = new FileInputStream(f);
			n = in.read(buf, 0, sizeof buf);
			if (n > 0) {
				StringBuffer strBuf;
				strBuf.append(buf, 0, n);
				String str = new String(strBuf);
				replace(workBuf, "</body>", str);
			}
		}

		String authorHome = Version.authorHome;
		replace(workBuf, "@AUTHORHOME@", authorHome);

		if (workBuf.indexOf("<!-- yyyy-mm-dd -->") >= 0) {
			time_t t;
			time(&t);
			struct tm *tm = localtime(&t);
			String year = toString(1900 + tm->tm_year, 4);
			String month = toString(tm->tm_mon + 1, 2);
			String day = toString(tm->tm_mday, 2);
			String date = "";
			date = date + year + "-" + month + "-" + day;
			replace(workBuf, "<!-- yyyy-mm-dd -->", date);
		}

		if (workBuf.indexOf("<!-- time -->") >= 0) {
			time_t t;
			time(&t);
			struct tm *tm = localtime(&t);
			String hour = toString(tm->tm_hour, 2);
			String min = toString(tm->tm_min, 2);
			String sec = toString(tm->tm_sec, 2);
			String time = "";
			time = time + hour + ":" + min + ":" + sec;
			replace(workBuf, "<!-- time -->", time);
		}

		if (config.getLogoURL() != NULL) {
			String logo = "";
			logo = logo
				+ "<img align=\"right\" src="
				+ "\"" + config.getLogoURL() + "\">";
			replace(workBuf, "<!-- logo -->", logo);
		}

		if (host.length() > 0)
			replace(workBuf, "@SERVER@", host);

		StringBuffer b;
		String port = new String(b + config.getPortNo());
		replace(workBuf, "@PORT@", port);

		String product = Version.product;
		replace(workBuf, "@PRODUCT@", product);

		String projectHome = Version.projectHome;
		replace(workBuf, "@PROJECTHOME@", projectHome);

		String version = Version.version;
		replace(workBuf, "@VERSION@", version);

		String entity = new String(workBuf);
		String type;
		if (file.getPath().endsWith(".html"))
			type = "text/html";
		else
			type = "application/x-java-jnlp-file";
		response.setEntity(entity, type);
	} else
		response.setEntity(file);

	if (method.equalsIgnoreCase("HEAD"))
		response.writeHead(writer);
	else
		response.write(writer);
}

String *HTTPServer::toString(int n, int digits) {
	StringBuffer str;
	str.append(n);
	while (str.length() < digits)
		str.insert(0, '0');

	return new String(str);
}
